#!/usr/bin/env python2.5
# -*- coding: utf-8 -*-

# copyright: m. dietrich
# license: gpl
__revision = '$Rev: 242 $'

from urllib2 import Request, urlopen
from struct import pack, unpack, calcsize
from gdbm import open as db_open
from base import log_info, log_debug, config, LOG_ERR, LOG_WARNING, LOG_INFO, LOG_DEBUG, LOG, dedbusmap
from freesmartphone import *

'''
dbus-send --system --print-reply --type=method_call --dest=org.mobile /org/mobile/GoogleLocation org.mobile.Location.SetPosition string:262.2.144.26993
'''

class GoogleLocation(NotifyObject):
	def __init__(self, bus):
		self.bus = bus
		NotifyObject.__init__(self,
			object_path='/org/mobile/GoogleLocation',
			bus_name=BusName(DBUS_NAME, self.bus),
			)
		self.latitude = 0.0
		self.longitude = 0.0
		self.cellid = None
		self.fix = False
		try: self.db = db_open(config.get('ggl', 'database'), 'wf')
		except: self.db = db_open(config.get('ggl', 'database'), 'cf')

	@method(DIN_LOCATION, 's', 'a{sv}')
	def SetPosition(self, cellid):
		LOG(LOG_DEBUG, __name__, 'SetPosition')
		if self.cellid != cellid:
			mcc, mnc, lac, ci, = [int(n) for n in cellid.split('.')]
			key = pack('!iiii', mcc, mnc, lac, ci, )
			if self.db.has_key(key):
				LOG(LOG_INFO, __name__, '__locate', 'from db')
				self.latitude, self.longitude = [float(n) / 1000000.0 for n in unpack('!ii', self.db[key])]
				self.fix = True
			else:
				self.fix, self.latitude, self.longitude = GoogleLocation.__locate(mcc, mnc, lac, ci, )
				self.cellid = cellid
				self.db[key] = pack('!ii', int(self.latitude*1000002), int(self.longitude*1000000))
		content = dict(latitude=self.latitude, longitude=self.longitude, fix=self.fix and 1 or 0, )
		self.Position(content)
		return content

	@notify(DIN_LOCATION, 'a{sv}')
	def Position(self, content):
		LOG(LOG_DEBUG, __name__, 'Position', content)

	@staticmethod
	def __locate(mcc, mnc, lac, ci):
		LOG(LOG_DEBUG, __name__, '__locate', lac, ci)
		try:
			country = 'en'
			user_agent = 'Sony_Ericsson-K810'
			version = '1.3.1'
			media = 'Web'
			type_field_list = (
				('h', 21), # hard coded guess it's the opcode
				('q', 0), # session id ?
				('h', len(country)),
				('%ds'% (len(country)), country), # country
				('h', len(user_agent)),
				('%ds'%(len(user_agent)), user_agent), #UA
				('h', len(version)),
				('%ds'%(len(version)), version), #version
				('h', len(media)),
				('%ds'%(len(media)), media), # dunno
				('b', 27), # dunno
				('i', 0), #(2);//MNC
				('i', 0), #(262);//MCC
				('i', 3), # Network 3 = GSM / 5 = other / -1 = unknown
				('h', 0),
				('0s', ''),
				('i', ci), # CELL-ID
				('i', lac), # LAC
				('i', 0),
				('i', 0),
				('i', 0),
				('i', 0),
				)
			body = pack(
				'!%s'% ''.join([type_field[0] for type_field in type_field_list]),
				*[type_field[1] for type_field in type_field_list])
			headers = {
				'Content-Type': 'application/binary',
				'Host': 'www.google.com',
				'Cache-Control': 'no-cache',
				'Pragma': 'no-cache',
				'User-Agent': 'Java/1.5.0_13',
				'Accept': 'text/html, image/gif, image/jpeg, *; q=.2, */*; q=.2',
				'Connection': 'keep-alive',
				}
			data = urlopen(Request('http://www.google.com/glm/mmap', body, headers)).read()
			_, _, status = unpack('!hbi', data[:7])
			if status != 0:
				raise Exception('google returned %d'% status)
			data = data[7:]
			latitude, longitude, _, _, _ = unpack('!iiiih', data)
			latitude /= 1000000.0
			longitude /= 1000000.0
			LOG(LOG_DEBUG, __name__, '__locate', latitude, longitude)
			return True, latitude, longitude
		except Exception, e:
			LOG(LOG_ERR, __name__, 'google error', e)
		return False, 0.0, 0.0

if __name__ == '__main__':
	from syslog import openlog, syslog, closelog, LOG_ERR, LOG_WARNING, LOG_INFO, LOG_DEBUG, LOG_DAEMON, LOG_NDELAY, LOG_PID, LOG_PERROR
	from gobject import MainLoop
	from dbus.mainloop.glib import DBusGMainLoop
	openlog('pyggld', LOG_NDELAY|LOG_PID|LOG_PERROR, LOG_DAEMON,)
	DBusGMainLoop(set_as_default=True)
	mainloop = MainLoop()
	daemon = GoogleLocation(InitBus())
	mainloop.run()
	closelog()
# vim:tw=0:nowrap
