#!/usr/bin/env python2.5
# -*- coding: utf-8 -*-

# copyright: m. dietrich
# license: gpl
__revision = '$Rev: 242 $'

from math import modf
from datetime import datetime
from socket import socket, AF_INET, SOCK_DGRAM
import serial
from time import sleep
from gobject import io_add_watch, IO_IN
from base import log_info, log_debug, config, LOG_ERR, LOG_WARNING, LOG_INFO, LOG_DEBUG, LOG
from freesmartphone import *

# dbus-send --system --print-reply --type=method_call --dest=org.mobile /org/mobile/GpsLocation org.mobile.Location.GetPosition

class GpsLocation(NotifyObject):
	def __init__(self, bus):
		NotifyObject.__init__(self,
			object_path='/org/mobile/GpsLocation',
			bus_name=BusName(DBUS_NAME, bus),
			)
		self.log = None
		self.dialect = []
		self.last_error = Empty
		self._position = dict(fix='0', )
		self._satellites = dict()
		self._dop = dict()
		udp_port = config.get('gps', 'udp_port')
		if udp_port:
			self.start_udp_receiver(int(udp_port))
		else:
			serial_device = config.get('gps', 'serial_device')
			if serial_device:
				self.start_serial_reader(serial_device)

	def handle_line(self, line):
		if line[0] != '$':
			raise Exception('Not NMEA: "%s"'% line)
		line = line[1:].strip().split('*')
		if len(line) != 2:
			raise Exception('No checksum: "%s"'% line)
		if not self.__chk_checksum(*line):
			raise Exception('Bad checksum: "%s"'% line)
		words = line[0].split(',')
		word = words.pop(0)
		if not word in self.dialect:
			self.dialect.append(word)
		if not GpsLocation.__dict__.has_key('gps'+word):
			raise Exception('Unknown sentence %s'% word)
		GpsLocation.__dict__['gps'+word](self, *words)

	def start_udp_receiver(self, udp_port):
		self.s = socket(AF_INET, SOCK_DGRAM)
		self.s.bind(('127.0.0.1', udp_port))
		self.iow = io_add_watch(self.s, IO_IN, self.__udp_read)

	def __udp_read(self, source, condition):
		try:
			line = self.s.recv(2048)
			if self.log: self.log.write(line)
			try:
				self.handle_line(line)
			except Exception, e:
				LOG(LOG_INFO, __name__, line, e)
				#assert line.startswith('$GP')
				#line = line[3:] # header
				#line = line.split('*')[0] # checksum
				#line = line.split(',') # fields
				#LOG(LOG_DEBUG, __name__, line, e)
			#LOG(LOG_DEBUG, __name__, self.__dict__)
		except Exception, e:
			import traceback
			traceback.print_exc()
			self.last_error = str(e)
			LOG(LOG_ERR, __name__, 'error', e)
		return True

	def start_file_reader(self, filename):
		self.s = open(filename, 'r')
		timeout_add(1000, self.__file_read)

	def __file_read(self):
		line = self.s.readline()
		if line:
			self.handle_line(line)
			return True
		else:
			self.s.close()
			return False

	def start_serial_reader(self, serial_device):
		self.__neo1973_on()
		self.s = serial.Serial()
		self.s.port = serial_device
		#self.s.baudrate = 115200
		self.s.timeout = 7
		#self.s.writeTimeout = 1
		self.s.xonxoff = False
		self.s.rtscts = True
		self.s.open()
		self.iow = io_add_watch(self.s, IO_IN, self.__serial_read)
		self.s.write(self.__add_checksum('PUBX,40,GLL,1,5,0,0'))
		self.s.write(self.__add_checksum('PUBX,40,VTG,1,5,0,0'))
		self.s.write(self.__add_checksum('PUBX,40,ZGA,1,5,0,0'))

	def __serial_read(self, source, condition):
		try:
			line = self.s.readline().strip()
			if line:
				if self.log: self.log.write(line)
				try:
					self.handle_line(line)
				except Exception, e:
					LOG(LOG_INFO, __name__, line, e)
					#assert line.startswith('$GP')
					#line = line[3:] # header
					#line = line.split('*')[0] # checksum
					#line = line.split(',') # fields
					#LOG(LOG_DEBUG, __name__, line, e)
				#LOG(LOG_DEBUG, __name__, self.__dict__)
		except Exception, e:
			import traceback
			traceback.print_exc()
			self.last_error = str(e)
			LOG(LOG_ERR, __name__, 'error', e)
		return True

	def __neo1973_hardware(self, name, value):
		'''
		write state to the sys file system to control modem hw
		'''
		fn = '/sys/bus/platform/devices/neo1973-pm-gps.0/%s'% name
		f = open(fn, 'w')
		try:
			f.write(value)
			LOG(LOG_DEBUG, __name__, 'system echo %s > %s'% (value, fn))
		finally:
			f.close()

	def __neo1973_off(self):
		#self.__neo1973_hardware('reset', '0')
		self.__neo1973_hardware('pwron', '0')

	def __neo1973_on(self):
		self.__neo1973_off()
		sleep(1.0)
		self.__neo1973_hardware('pwron', '1')
		#sleep(1.0)
		#LOG(LOG_DEBUG, __name__, 'system sleep', 1)
		#self.__neo1973_hardware('reset', '1')
		#sleep(1.0)
		#LOG(LOG_DEBUG, __name__, 'system sleep', 1)
		#self.__neo1973_hardware('reset', '0')
		#sleep(2.0)
		#LOG(LOG_DEBUG, __name__, 'system sleep', 2)

	def get_status(self,satellite):
		if self.ZCHseen:
			for i in range(12):
				if satellite == self.zs[i]:
					return (self.zv[i] & 7) | 8
			return 0
		else:
			for i in range(12):
				if satellite == self.prn[i]:
					s = self.ss[i] / 6
					return min(s, 7)
			return 0

	def get_speed(self, speed):
		if not speed: return speed
		return speed * 1.852

	def get_signal_strength(self):
		return self.in_view and (float(sum(self.ss[:self.in_view])) / self.in_view) or 0.0

	@method(DIN_LOCATION, 's', 'a{sv}')
	def SetPosition(self, cellid):
		raise Exception('not implemented')

	@notify(DIN_LOCATION, 'a{sv}')
	def Position(self, _position):
		LOG(LOG_DEBUG, __name__, 'Position', _position)
		self._position = _position

	@method(DIN_LOCATION, '', 'a{sv}')
	def GetPosition(self):
		return self._position

	@notify(DIN_LOCATION, 'a{sv}')
	def Satellites(self, _satellites):
		LOG(LOG_DEBUG, __name__, 'Satellites', _satellites)
		self._satellites = _satellites

	@method(DIN_LOCATION, '', 'a{sv}')
	def GetSatellites(self):
		return self._satellites

	@notify(DIN_LOCATION, 'a{sv}')
	def Accuracy(self, _dop):
		LOG(LOG_DEBUG, __name__, 'Accuracy', _dop)
		self._dop = _dop

	@method(DIN_LOCATION, '', 'a{sv}')
	def GetAccuracy(self):
		return self._dop

	@method(DIN_LOCATION, 's', '')
	def SetNmeaLog(self, filename):
		if self.log:
			self.log.close()
		if filename:
			self.log = open(filename, 'a')

	@method(DIN_LOCATION, 's', '')
	def SetNmeaPlayback(self, filename):
		self.start_file_reader(filename)

	def __add_checksum(self, sentence):
		csum = 0
		sentence = sentence.strip().split('*')[0]
		for c in sentence:
			csum = csum ^ ord(c)
		return '$%s*%02X\r\n'% (sentence, csum, )

	def __chk_checksum(self,sentence, cksum):
		csum = 0
		for c in sentence:
			csum = csum ^ ord(c)
		return '%02X' % csum == cksum

	def __do_lat_lon(self, l, h):
		if not l:
			return l
		if l[-1] in ('N', 'S', 'E', 'W', ):
			h = l[-1]
			l = l[:-1]
		l = float(l)
		frac, intpart = modf(l / 100.0)
		l = intpart + frac * 100.0 / 60.0
		if h in ('S', 'W', ):
			l = -l
		return l

	def gpsGPGGA(self, time, latitude, ln, longitude, lw, fix, satellites, dilution, altitude, *words):
		'''
			GGA - Global Positioning System Fix Data
			123519,4807.038,N,01131.324,E,1,08,0.9,545.4,M,46.9,M, , 
				123519 Fix taken at 12:35:19 UTC
				4807.038,N Latitude 48 deg 07.038' N
				01131.324,E Longitude 11 deg 31.324' E
				1 Fix quality: 0 = invalid 1 = GPS fix 2 = DGPS fix
				08 Number of satellites being tracked
				0.9 Horizontal dilution of position
				545.4,M Altitude, Metres, above mean sea level
				46.9,M Height of geoid (mean sea level) above WGS84 ellipsoid
				(empty field) time in seconds since last DGPS update
				(empty field) DGPS station ID number
			'''
		fix = int(fix)
		if fix > 0:
			_position = dict(self._position,
				fix=fix,
				latitude = self.__do_lat_lon(latitude, ln),
				longitude = self.__do_lat_lon(longitude, lw),
				altitude = float(altitude),
				satellites = int(satellites),
				)
		else:
			_position = dict(self._position,
				fix=fix,
				)
		if self._position != _position:
			self.Position(_position)

	def gpsGPRMC(self, gpstime, warning, latitude, ln, longitude, lw, speed, course, gpsdate, magneticvariation, me, mw):
		'''
			RMC - Recommended minimum specific GPS/Transit data
			225446,A,4916.45,N,12311.12,W,000.5,054.7,191194,020.3,E
				225446 Time of fix 22:54:46 UTC
				A Navigation receiver warning A = OK, V = warning
				4916.45
				N Latitude 49 deg. 16.45 min North
				12311.12
				W Longitude 123 deg. 11.12 min West
				000.5 Speed over ground, Knots
				054.7 Course Made Good, True
				191194 Date of fix 19 November 1994
				????
				020.3
				E Magnetic variation 20.3 deg East
			'''
		if warning == 'A': fix = 1
		else: fix = 0
		_position = dict(self._position,
			warning=warning,
			fix=fix,
			)
		if gpsdate and gpstime:
			_position['time'] = datetime(
				2000 + int(gpsdate[4:6]),
				int(gpsdate[2:4]),
				int(gpsdate[0:2]),
				int(gpstime[0:2]),
				int(gpstime[2:4]),
				int(gpstime[4:6]),
				).isoformat()
		if latitude and ln and longitude and lw:
			_position['latitude'] = self.__do_lat_lon(latitude, ln)
			_position['longitude'] = self.__do_lat_lon(longitude, lw)
		if speed:
			speed = float(speed)
			_position['knots'] = speed
			_position['kph'] = self.get_speed(speed)
		if course:
			_position['course'] = float(course)
		if self._position != _position:
			self.Position(_position)

	def gpsGPGSV(self, numbersentences, sentence, in_view, *words):
		'''
			GSV - Satellites in view
			2,1,08,01,40,083,46,02,17,308,41,12,07,344,39,14,22,228,45
				2 Number of sentences for full data
				1 sentence 1 of 2
				08 Number of satellites in view

				01 Satellite PRN number
				40 Elevation, degrees
				083 Azimuth, degrees
				46 Signal strength - higher is better
				<repeat for up to 4 satellites per sentence>
				There my be up to three GSV sentences in a data packet
			'''
		sentence = int(sentence)
		numbersentences = int(numbersentences)
		in_view = int(in_view)
		words = list(words)
		if sentence == 1:
			self._satellites = dict(
				numbersentences = numbersentences,
				in_view = in_view,
				)
		while words:
			prn = words.pop(0)
			elevation = words.pop(0)
			azimuth = words.pop(0)
			ss = words.pop(0)
			self._satellites[prn] = {}
			if azimuth:
				self._satellites[prn]['azimuth'] = int(azimuth)
			if elevation:
				self._satellites[prn]['elevation'] = int(elevation)
			if ss:
				self._satellites[prn]['ss'] = int(ss)
		if sentence == numbersentences:
			if self._satellites != self._satellites:
				self.Satellites(self._satellites)

	def gpsGPGSA(self, selection, mode, s0, s1, s2, s3, s4, s5, s6, s7, s8, s9, s10, s11, pdop, hdop, vdop):
		'''
			GSA - GPS DOP and active satellites
			A,3,04,05,,09,12,,,24,,,,,2.5,1.3,2.1
				A Auto selection of 2D or 3D fix (M = manual)
				3 3D fix
				04,05... PRNs of satellites used for fix (space for 12)
				2.5 PDOP (dilution of precision)
				1.3 Horizontal dilution of precision (HDOP)
				2.1 Vertical dilution of precision (VDOP)
			DOP is an indication of the effect of satellite geometry on
			the accuracy of the fix.

			http://www.kowoma.de/en/gps/errors.htm
			'''
		_dop = dict(
			mode = float(mode),
			# Geometric Dilution Of Precision: should not be smaller than 5
			pdop = float(pdop), # Positional Dilution Of Precision
			hdop = float(hdop), # Horizontal Dilution Of Precision: below 4 are good, above 8 bad
			vdop = float(vdop), # Vertical Dilution Of Precision
			# Time Dilution Of Precision
			)
		if self._dop != _dop:
			self.Accuracy(_dop)

	def gpsGPTXT(self, *words):
		'''
		'''
		LOG(LOG_DEBUG, __name__, 'gpsGPTXT', words)

	def gpsGPVTG(self, cog, cogu, cogm, cogmu, sog, sogu, kph, kphu, mode):
		'''
		$GPVTG,77.52,T,,M,0.004,N,0.008,K,A*06
			Course over ground (true) 
			T fixed field 
			Course over ground (magnetic). 
			M fixed field 
			Speed over ground 
			N Knots - fixed field 
			km/h 
			K Kilometers per hour - fixed field 
			Mode Indicator 
		'''
		LOG(LOG_DEBUG, __name__, 'gpsGPVTG', cog, cogu, cogm, cogmu, kph, kphu, mode)
		_position = dict(self._position,
			warning = mode,
			)
		if mode == 'A':
			if sog:
				_position['knots'] = float(sog)
			if kph:
				_position['kph'] = float(kph)
			if cog:
				_position['cog'] = float(cog)
		if self._position != _position:
			self.Position(_position)

	def gpsGPGLL(self, latitude, ln, longitude, lw, time, valid, mode):
		'''
		'''
		LOG(LOG_DEBUG, __name__, 'gpsGPGLL', latitude, ln, longitude, lw, valid, mode)
		_position = dict(self._position,
			warning = mode,
			)
		if valid == 'A':
			_position = dict(self._position)
			if latitude and ln and longitude and lw:
				_position['latitude'] = self.__do_lat_lon(latitude, ln)
				_position['longitude'] = self.__do_lat_lon(longitude, lw)
		if self._position != _position:
			self.Position(_position)

	def gpsGPZDA(self, time, day, month, year, ltzh, ltzn):
		'''
		'''
		LOG(LOG_DEBUG, __name__, 'gpsGPZDA', time, day, month, year, ltzh, ltzn)

	def gpsPGLOR(self, *words):
		'''
		'''
		LOG(LOG_DEBUG, __name__, 'gpsPGLOR', words)

if __name__ == '__main__':
	from syslog import openlog, syslog, closelog, LOG_ERR, LOG_WARNING, LOG_INFO, LOG_DEBUG, LOG_DAEMON, LOG_NDELAY, LOG_PID, LOG_PERROR
	from gobject import MainLoop
	from dbus.mainloop.glib import DBusGMainLoop
	openlog('pygpsd', LOG_NDELAY|LOG_PID|LOG_PERROR, LOG_DAEMON,)
	DBusGMainLoop(set_as_default=True)
	mainloop = MainLoop()
	daemon = GpsLocation(InitBus())
	mainloop.run()
	closelog()
# vim:tw=0:nowrap
