#!/usr/bin/env python2.5
# -*- coding: utf-8 -*-

# copyright: m. dietrich
# license: gpl
__revision = '$Rev: 255 $'

from time import time
from os import system, kill, chmod
from os.path import exists
from stat import S_IRWXU, S_IRWXG, S_IRWXO
from signal import SIGINT, signal
from time import sleep
from base import log_info, log_debug, config, LOG_ERR, LOG_WARNING, LOG_INFO, LOG_DEBUG, LOG
from gobject import timeout_add, child_watch_add, spawn_async
from freesmartphone import *

'''
DB="dbus-send --system --print-reply --type=method_call --dest=org.mobile /org/mobile/GprsNetwork"
$DB org.mobile.Network.Activate boolean:true
'''

# see http://www.phonestar.com.my/s_at_10.html
#'OK' 'AT+CGDPADDR=1'\
#'OK' 'AT+CGDATA="ppp"'\
#'OK' 'AT+CGACT=1,1'\
#'OK' 'AT+CGATT=1'\
#'OK' 'AT+CFUN=1'\
#'OK' 'AT+COPS?'\
#'OK' 'AT+CREG?'\
#'OK' 'AT+CPAS'\

class Gprs(NotifyObject):
	__init_filename = '/var/tmp/gprs-connect-chat'
	__hangup_filename = '/var/tmp/gprs-disconnect-chat'
	__patch_files = {
		__init_filename: r'''#!/bin/sh -e
exec /usr/sbin/chat -v\
	'ABORT' 'BUSY'\
	'ABORT' 'DELAYED'\
	'ABORT' 'ERROR'\
	'ABORT' 'NO ANSWER'\
	'ABORT' 'NO CARRIER'\
	'ABORT' 'NO DIALTONE'\
	'ABORT' 'RINGING'\
	'ABORT' 'VOICE'\
	'TIMEOUT' '3'\
	'' 'ATZ'\
	'OK-\k\k\k\d+++ATH-OK' 'ATE0'\
	'OK' 'AT+CMEE=2'\
	'OK' 'AT+CPIN?'\
	'READY' '\c'\
	'OK' 'AT+CGDCONT=1,"IP","%s"'\
	'TIMEOUT' '180'\
	'OK' 'ATD*99#'\
	'CONNECT' '\d\c'
'''% config.get('ppp', 'apn'),
		'/etc/ppp/chap-secrets': '* * "%s" *\n'% '',
		'/etc/ppp/pap-secrets': '* * "%s" *\n'% '',
		'/etc/ppp/ip-up.d/08setupdns': '''#!/bin/sh -e
cp /var/run/ppp/resolv.conf /etc/resolv.conf
''',
		'/etc/ppp/ip-down.d/92removedns': '''#!/bin/sh -e
echo nameserver 127.0.0.1 > /etc/resolv.conf
''',
		__hangup_filename: r'''#!/bin/sh -e
exec /usr/sbin/chat -v\
	'ABORT' 'OK'\
	'ABORT' 'BUSY'\
	'ABORT' 'DELAYED'\
	'ABORT' 'NO ANSWER'\
	'ABORT' 'NO CARRIER'\
	'ABORT' 'NO DIALTONE'\
	'ABORT' 'VOICE'\
	'ABORT' 'ERROR'\
	'ABORT' 'RINGING'\
	'TIMEOUT' '60'\
	'' '\k\k\k\d+++ATH'\
	'NO CARRIER-AT-OK' ''
''',
	}
	__pppd_command = '/usr/sbin/pppd'
	__pppd_options = [
		'115200',
		'nodetach',
		'connect', __init_filename,
		'crtscts',
		'defaultroute',
		'debug',
		'disconnect', __hangup_filename,
		'hide-password',
		'holdoff', '3',
		'ipcp-accept-local',
		'ktune',
		'lcp-echo-failure', '8',
		'lcp-echo-interval', '3',
		'ipcp-max-configure', '32',
		'lock',
		'noauth',
		#'demand',
		'noipdefault',
		'novj',
		'novjccomp',
		#'persist',
		'proxyarp',
		'replacedefaultroute',
		'usepeerdns',
		]

	def __init__(self, bus):
		self.bus = bus
		NotifyObject.__init__(self,
			object_path='/org/mobile/GprsNetwork',
			bus_name=BusName(DBUS_NAME, self.bus),
			)
		self.cpid = -1
		self.port = ''
		self.last_status = dict(device='')
		timeout_add(12 * 1000, self.__poll)

	def __child_watch(self, pid, condition, user_data):
		print '__child_watch', pid, condition, user_data

	@method(DIN_NETWORK, 'b', '')
	def Activate(self, on):
		LOG(LOG_DEBUG, __name__, 'Activate')
		try:
			if on:
				if self.cpid >= 0: raise Exception('already active')
				muxer = self.bus.get_object(config.get('ppp', 'bus'), config.get('ppp', 'obj'))
				muxer = Interface(muxer, 'org.freesmartphone.GSM.MUX')
				self.port = str(muxer.AllocChannel('ppp'))
				del muxer
				if not self.port: raise Exception('no device')
				# go direct self.port = '/dev/ttySAC0'
				LOG(LOG_INFO, __name__, 'Activate got port', self.port)
				for filename, content in Gprs.__patch_files.items():
					LOG(LOG_DEBUG, __name__, 'change file for our needs:', filename);
					f = open(filename, 'w')
					f.write(content)
					f.close()
					chmod(filename, S_IRWXU|S_IRWXO|S_IRWXO)
				self.cpid, _, _, _ = spawn_async(
					[Gprs.__pppd_command, self.port, ] + Gprs.__pppd_options,
					standard_input=False, standard_output=False, standard_error=False,
					#dict(PATH='/bin:/usr/bin:/sbin:/usr/sbin', ),
					)
				child_watch_add(self.cpid, self.__child_watch)
			else:
				if self.cpid < 0: raise Exception('already inactive')
				try:
					LOG(LOG_INFO, __name__, 'Activate kill pppd pid', self.cpid)
					kill(self.cpid, SIGINT)
					#LOG(LOG_INFO, __name__, 'Activate waiting for pppd')
					#p, r = waitpid(self.cpid, 0)
					#LOG(LOG_INFO, __name__, 'Activate pppd returned', r)
					if exists(self.port):
						try: open(self.port, 'rw').close()
						except: pass
				finally:
					self.cpid = -1
				system('ifdown usb0')
				system('ifup usb0')
		except Exception, e:
			LOG(LOG_ERR, __name__, 'Activate', e)

	@notify(DIN_NETWORK, 'a{sv}')
	def Status(self, newmap):
		self.last_status = newmap
		LOG(LOG_INFO, __name__, 'Status', self.last_status)

	@method(DIN_NETWORK, '', 'a{sv}')
	def GetStatus(self):
		return self.last_status
		LOG(LOG_INFO, __name__, 'GetStatus', self.last_status)

	def get_default_route(self):
		f = open('/proc/net/route', 'r')
		l = f.readlines()
		f.close()
		for n in l:
			n = n.split('\t')
			if n[1] == '00000000':
				return n[0]
		return ''

	def __poll(self):
		status = dict(device=self.get_default_route())
		LOG(LOG_DEBUG, __name__, '__poll', status, self.last_status)
		if self.last_status != status:
			self.Status(status)
		return True

if __name__ == '__main__':
	from syslog import openlog, syslog, closelog, LOG_ERR, LOG_WARNING, LOG_INFO, LOG_DEBUG, LOG_DAEMON, LOG_NDELAY, LOG_PID, LOG_PERROR
	from gobject import MainLoop
	from dbus.mainloop.glib import DBusGMainLoop
	openlog('pypppd', LOG_NDELAY|LOG_PID|LOG_PERROR, LOG_DAEMON,)
	DBusGMainLoop(set_as_default=True)
	mainloop = MainLoop()
	a = Gprs(InitBus())
	mainloop.run()
	closelog()
#import time
#import gobject
#gobject.threads_init()
#def iowatch(source, condition):
#    print "iowatch"
#    return True
#   
#def childwatch(pid, condition):
#    print "childwatch"
#    return True
#
#def timeout():
#    print "bla"
#    return True
#
#pid, stdin, stdout, stderr = gobject.spawn_async(["child.py"],standard_input=False, standard_output=True, standard_error=False)
#gobject.child_watch_add(pid, childwatch)
#gobject.io_add_watch(stdout, gobject.IO_IN, iowatch)
#gobject.timeout_add(1000, timeout)
#print "running"
#mainloop = gobject.MainLoop()
#mainloop.run()
# vim:tw=0:nowrap
