#!/usr/bin/env python2.5
# -*- coding: utf-8 -*-

# copyright: m. dietrich
# license: gpl
__revision = '$Rev: 218 $'

from os import system
from gobject import timeout_add
from base import log_info, log_debug, config, LOG_ERR, LOG_WARNING, LOG_INFO, LOG_DEBUG, LOG
from freesmartphone import *

# dbus-send --system --print-reply --type=method_call --dest=org.mobile /org/mobile/Power org.mobile.Power.GetStatus

class Power(NotifyObject):
	# MAX = 4.235 4.100
	# MIN = 3.400 3.700
	def __init__(self, bus):
		NotifyObject.__init__(self,
			object_path='/org/mobile/Power',
			bus_name=BusName(DBUS_NAME, bus),
			)
		self._status = {}
		self.max_brightness = self.get_max_brightness()
		timeout_add(9 * 1000, self.__poll)

	@method(DIN_POWER, 'b', '')
	def PowerOff(self, restart):
		if restart:
			system('reboot')
		else:
			system('halt')

	@method(DIN_POWER, '', 'a{sv}')
	def GetStatus(self):
		return self._status

	@notify(DIN_POWER, 'a{sv}')
	def Status(self, _status):
		LOG(LOG_INFO, __name__, 'Status', _status)
		self._status = _status

	def __reader(self, name):
		try: s = open('/sys/devices/platform/s3c2440-i2c/i2c-adapter/i2c-0/0-0073/%s'% name, 'r')
		except: s = open('/sys/devices/platform/s3c2410-i2c/i2c-adapter/i2c-0/0-0008/%s'% name, 'r')
		ret = s.readline().strip()
		s.close()
		if not ret: ret = '0'
		return ret

	def __apm(self):
		s = open('/proc/apm', 'r')
		ret = s.readline()
		s.close()
		#sscanf(buf, "%s %d.%d %x %x %x %x %d%% %d %s\n", version, &ignore, &ignore, &flags, &ac, &ignore2, &ignore2, &battery_percent, &battery_time, ignore3);
		return ret.strip().split()

	def __poll(self):
		try:
			apm = self.__apm()
			_status = dict(
				battemp = int(self.__reader('battemp')),
				battvolt = int(self.__reader('battvolt')) / 1000.0,
				chgmode = self.__reader('chgmode'),
				chgstate = self.__reader('chgstate'),
				chgcur = int(self.__reader('chgcur')) / 1000.0,
				)
			if self._status != _status:
				self.Status(_status)
		except Exception, e:
			LOG(LOG_ERR, __name__, '__poll', e)
		return True

	def get_max_brightness(self):
		try: f = open("/sys/class/backlight/pcf50633-bl/max_brightness", 'r')
		except: f = open("/sys/class/backlight/gta01-bl/max_brightness", 'r')
		l = f.readline()
		f.close()
		return int(l)

	@method(DIN_POWER, 'i', '')
	def set_brightness(self, percentage):
		val = self.max_brightness * 100 / percentage
		try: f = open("/sys/class/backlight/pcf50633-bl/brightness", 'w')
		except: f = open("/sys/class/backlight/gta01-bl/brightness", 'w')
		f.write(str(val))
		f.close()

	@method(DIN_POWER, 'i', '')
	def vibrate(self, length=None):
		LOG(LOG_DEBUG, __name__, 'vibrate', length)
		fd = open("/sys/class/leds/neo1973:vibrator/brightness", 'w')
		fd.write(length is None and "0" or "255")
		fd.close()
		if length is not None:
			timeout_add(length, self.vibrate)

if __name__ == '__main__':
	from syslog import openlog, syslog, closelog, LOG_ERR, LOG_WARNING, LOG_INFO, LOG_DEBUG, LOG_DAEMON, LOG_NDELAY, LOG_PID, LOG_PERROR
	from gobject import MainLoop
	from dbus.mainloop.glib import DBusGMainLoop
	openlog('pypwrd', LOG_NDELAY|LOG_PID|LOG_PERROR, LOG_DAEMON,)
	DBusGMainLoop(set_as_default=True)
	mainloop = MainLoop()
	a = Power(InitBus())
	mainloop.run()
	closelog()
# vim:tw=0:nowrap
#echo timer > /sys/class/leds/gta02-power\:blue/trigger
#echo 400 > /sys/class/leds/gta02-power\:blue/delay_on
#echo 1300 > /sys/class/leds/gta02-power\:blue/delay_off
